"""surfmon.

Monitor Windsurf and Windsurf Next resource usage and diagnose problems.
"""

from __future__ import annotations

from importlib.metadata import version

__all__: list[str] = []

__version__ = version("surfmon")
